@@ -27,62 +27,47 @@
 #include <common/compat/string.h>
 #include <lttng/constant.h>
 
-#include "cmd-generic.h"
-#include "cmd-2-1.h"
-#include "utils.h"
+#include "cmd-2-4.h"
+#include "lttng-relayd.h"
 
-/*
- * cmd_recv_stream_2_2 allocates path_name and channel_name.
- */
-int cmd_recv_stream_2_2(struct relay_connection *conn,
-		char **ret_path_name, char **ret_channel_name,
-		uint64_t *tracefile_size, uint64_t *tracefile_count)
+int cmd_create_session_2_4(const struct lttng_buffer_view *payload,
+		char *session_name, char *hostname,
+		uint32_t *live_timer, bool *snapshot)
 {
 	int ret;
-	struct lttcomm_relayd_add_stream_2_2 stream_info;
-	char *path_name = NULL;
-	char *channel_name = NULL;
+	struct lttcomm_relayd_create_session_2_4 session_info;
 	size_t len;
 
-	ret = cmd_recv(conn->sock, &stream_info, sizeof(stream_info));
-	if (ret < 0) {
-		ERR("Unable to recv stream version 2.2");
+	if (payload->size < sizeof(session_info)) {
+		ERR("Unexpected payload size in \"cmd_create_session_2_4\": expected >= %zu bytes, got %zu bytes",
+				sizeof(session_info), payload->size);
+		ret = -1;
 		goto error;
 	}
+	memcpy(&session_info, payload->data, sizeof(session_info));
 
-	len = lttng_strnlen(stream_info.pathname, sizeof(stream_info.pathname));
+	len = lttng_strnlen(session_info.session_name, sizeof(session_info.session_name));
 	/* Ensure that NULL-terminated and fits in local filename length. */
-	if (len == sizeof(stream_info.pathname) || len >= LTTNG_NAME_MAX) {
+	if (len == sizeof(session_info.session_name) || len >= LTTNG_NAME_MAX) {
 		ret = -ENAMETOOLONG;
-		ERR("Path name too long");
-		goto error;
-	}
-	path_name = create_output_path(stream_info.pathname);
-	if (!path_name) {
-		PERROR("Path name allocation");
-		ret = -ENOMEM;
-		goto error;
-	}
-	len = lttng_strnlen(stream_info.channel_name, sizeof(stream_info.channel_name));
-	if (len == sizeof(stream_info.channel_name) || len >= DEFAULT_STREAM_NAME_LEN) {
-		ret = -ENAMETOOLONG;
-		ERR("Channel name too long");
-		goto error;
-	}
-	channel_name = strdup(stream_info.channel_name);
-	if (!channel_name) {
-		ret = -errno;
-		PERROR("Channel name allocation");
+		ERR("Session name too long");
 		goto error;
 	}
+	strncpy(session_name, session_info.session_name, LTTNG_NAME_MAX);
+
+	len = lttng_strnlen(session_info.hostname, sizeof(session_info.hostname));
+	if (len == sizeof(session_info.hostname) || len >= LTTNG_HOST_NAME_MAX) {
+		ret = -ENAMETOOLONG;
+		ERR("Session name too long");
+		goto error;
+	}
+	strncpy(hostname, session_info.hostname, LTTNG_HOST_NAME_MAX);
+
+	*live_timer = be32toh(session_info.live_timer);
+	*snapshot = be32toh(session_info.snapshot);
+
+	ret = 0;
 
-	*tracefile_size = be64toh(stream_info.tracefile_size);
-	*tracefile_count = be64toh(stream_info.tracefile_count);
-	*ret_path_name = path_name;
-	*ret_channel_name = channel_name;
-	return 0;
 error:
-	free(path_name);
-	free(channel_name);
 	return ret;
 }
