@@ -26,23 +26,37 @@
 #include <urcu/list.h>
 
 #include <common/hashtable/hashtable.h>
+#include <common/trace-chunk.h>
+#include <common/optional.h>
+#include <common/buffer-view.h>
 
 #include "session.h"
 #include "stream-fd.h"
 #include "tracefile-array.h"
 
+struct lttcomm_relayd_index;
+
+struct relay_stream_rotation {
+	/*
+	 * Indicates if the stream's data and index have been rotated. A
+	 * rotation is considered completed when both rotations have occurred.
+	 */
+	bool data_rotated;
+	bool index_rotated;
+	/*
+	 * Sequence number of the first packet of the new trace chunk to which
+	 * the stream is rotating.
+	 */
+	uint64_t seq_num;
+	struct lttng_trace_chunk *next_trace_chunk;
+};
+
 /*
  * Represents a stream in the relay
  */
 struct relay_stream {
 	uint64_t stream_handle;
 
-	/*
-	 * reflock used to synchronize the closing of this stream.
-	 * stream reflock nests inside viewer stream reflock.
-	 * stream reflock nests inside index reflock.
-	 */
-	pthread_mutex_t reflock;
 	struct urcu_ref ref;
 	/* Back reference to trace. Protected by refcount on trace object. */
 	struct ctf_trace *trace;
@@ -53,8 +67,12 @@ struct relay_stream {
 	 * inside the ctf_trace lock.
 	 */
 	pthread_mutex_t lock;
-	uint64_t prev_seq;		/* previous data sequence number encountered. */
-	uint64_t last_net_seq_num;	/* seq num to encounter before closing. */
+	/* previous data sequence number written to disk. */
+	uint64_t prev_data_seq;
+	/* previous index sequence number written to disk. */
+	uint64_t prev_index_seq;
+	/* seq num to encounter before closing. */
+	uint64_t last_net_seq_num;
 
 	/* FD on which to write the stream data. */
 	struct stream_fd *stream_fd;
@@ -67,7 +85,23 @@ struct relay_stream {
 	/* On-disk circular buffer of tracefiles. */
 	uint64_t tracefile_size;
 	uint64_t tracefile_size_current;
+	/* Max number of trace files for this stream. */
 	uint64_t tracefile_count;
+	/*
+	 * Index of the currently active file for this stream's on-disk
+	 * ring buffer.
+	 */
+	uint64_t tracefile_current_index;
+	/*
+	 * Indicates that the on-disk buffer has wrapped around. Stream
+	 * files shall be unlinked before being opened after this has occurred.
+	 */
+	bool tracefile_wrapped_around;
+
+	/*
+	 * Position in the tracefile where we have the full index also on disk.
+	 */
+	uint64_t pos_after_last_complete_data_index;
 
 	/*
 	 * Counts the number of received indexes. The "tag" associated
@@ -109,7 +143,7 @@ struct relay_stream {
 	bool data_pending_check_done;
 
 	/* Is this stream a metadata stream ? */
-	int32_t is_metadata;
+	bool is_metadata;
 	/* Amount of metadata received (bytes). */
 	uint64_t metadata_received;
 
@@ -128,13 +162,20 @@ struct relay_stream {
 	 */
 	bool in_recv_list;
 	struct cds_list_head recv_node;
-	bool published;	/* Protected by session lock. */
+	/* Protected by session lock. */
+	bool published;
 	/*
 	 * Node of stream within global stream hash table.
 	 */
 	struct lttng_ht_node_u64 node;
 	bool in_stream_ht;		/* is stream in stream hash table. */
 	struct rcu_head rcu_node;	/* For call_rcu teardown. */
+	/*
+	 * The trace chunk to which the file currently being produced (if any)
+	 * belongs.
+	 */
+	struct lttng_trace_chunk *trace_chunk;
+	LTTNG_OPTIONAL(struct relay_stream_rotation) ongoing_rotation;
 };
 
 struct relay_stream *stream_create(struct ctf_trace *trace,
@@ -145,8 +186,28 @@ struct relay_stream *stream_create(struct ctf_trace *trace,
 struct relay_stream *stream_get_by_id(uint64_t stream_id);
 bool stream_get(struct relay_stream *stream);
 void stream_put(struct relay_stream *stream);
+int stream_rotate_output_files(struct relay_session *session,
+		struct relay_stream *stream);
+int stream_set_pending_rotation(struct relay_stream *stream,
+		struct lttng_trace_chunk *next_trace_chunk,
+		uint64_t rotation_sequence_number);
 void try_stream_close(struct relay_stream *stream);
 void stream_publish(struct relay_stream *stream);
+int stream_init_packet(struct relay_stream *stream, size_t packet_size,
+		bool *file_rotated);
+int stream_write(struct relay_stream *stream,
+		const struct lttng_buffer_view *packet, size_t padding_len);
+/* Called after the reception of a complete data packet. */
+int stream_update_index(struct relay_stream *stream, uint64_t net_seq_num,
+		bool rotate_index, bool *flushed, uint64_t total_size);
+int stream_complete_packet(struct relay_stream *stream,
+		size_t packet_total_size, uint64_t sequence_number,
+		bool index_flushed);
+/* Index info is in host endianness. */
+int stream_add_index(struct relay_stream *stream,
+		const struct lttcomm_relayd_index *index_info);
+int stream_reset_file(struct relay_stream *stream);
+
 void print_relay_streams(void);
 
 #endif /* _STREAM_H */
