@@ -23,6 +23,20 @@
 
 struct notification_thread_handle;
 
+/*
+ * A callback (and associated user data) that should be run after a command
+ * has been executed. No locks should be taken while executing this handler.
+ *
+ * The command's reply should not be sent until the handler has run and
+ * completed successfully. On failure, the handler's return code should
+ * be the only reply sent to the client.
+ */
+typedef enum lttng_error_code (*completion_handler_function)(void *);
+struct cmd_completion_handler {
+	completion_handler_function run;
+	void *data;
+};
+
 /*
  * Init the command subsystem. Must be called before using any of the functions
  * above. This is called in the main() of the session daemon.
@@ -30,17 +44,17 @@ struct notification_thread_handle;
 void cmd_init(void);
 
 /* Session commands */
-int cmd_create_session_uri(char *name, struct lttng_uri *uris,
-		size_t nb_uri, lttng_sock_cred *creds, unsigned int live_timer);
-int cmd_create_session_snapshot(char *name, struct lttng_uri *uris,
-		size_t nb_uri, lttng_sock_cred *creds);
-int cmd_destroy_session(struct ltt_session *session, int wpipe);
+enum lttng_error_code cmd_create_session(struct command_ctx *cmd_ctx, int sock,
+		struct lttng_session_descriptor **return_descriptor);
+int cmd_destroy_session(struct ltt_session *session,
+		struct notification_thread_handle *notification_thread_handle,
+		int *sock_fd);
 
 /* Channel commands */
 int cmd_disable_channel(struct ltt_session *session,
 		enum lttng_domain_type domain, char *channel_name);
 int cmd_enable_channel(struct ltt_session *session,
-		struct lttng_domain *domain, struct lttng_channel *attr,
+		const struct lttng_domain *domain, const struct lttng_channel *attr,
 		int wpipe);
 int cmd_track_pid(struct ltt_session *session, enum lttng_domain_type domain,
 		int pid);
@@ -50,14 +64,14 @@ int cmd_untrack_pid(struct ltt_session *session, enum lttng_domain_type domain,
 /* Event commands */
 int cmd_disable_event(struct ltt_session *session,
 		enum lttng_domain_type domain,
-		char *channel_name,
-		struct lttng_event *event);
+		const char *channel_name,
+		const struct lttng_event *event);
 int cmd_add_context(struct ltt_session *session, enum lttng_domain_type domain,
-		char *channel_name, struct lttng_event_context *ctx, int kwpipe);
+		char *channel_name, const struct lttng_event_context *ctx, int kwpipe);
 int cmd_set_filter(struct ltt_session *session, enum lttng_domain_type domain,
 		char *channel_name, struct lttng_event *event,
 		struct lttng_filter_bytecode *bytecode);
-int cmd_enable_event(struct ltt_session *session, struct lttng_domain *domain,
+int cmd_enable_event(struct ltt_session *session, const struct lttng_domain *domain,
 		char *channel_name, struct lttng_event *event,
 		char *filter_expression,
 		struct lttng_filter_bytecode *filter,
@@ -86,8 +100,8 @@ ssize_t cmd_list_channels(enum lttng_domain_type domain,
 		struct ltt_session *session, struct lttng_channel **channels);
 ssize_t cmd_list_domains(struct ltt_session *session,
 		struct lttng_domain **domains);
-void cmd_list_lttng_sessions(struct lttng_session *sessions, uid_t uid,
-		gid_t gid);
+void cmd_list_lttng_sessions(struct lttng_session *sessions,
+		size_t session_count, uid_t uid, gid_t gid);
 ssize_t cmd_list_tracepoint_fields(enum lttng_domain_type domain,
 		struct lttng_event_field **fields);
 ssize_t cmd_list_tracepoints(enum lttng_domain_type domain,
@@ -102,11 +116,11 @@ int cmd_data_pending(struct ltt_session *session);
 
 /* Snapshot */
 int cmd_snapshot_add_output(struct ltt_session *session,
-		struct lttng_snapshot_output *output, uint32_t *id);
+		const struct lttng_snapshot_output *output, uint32_t *id);
 int cmd_snapshot_del_output(struct ltt_session *session,
-		struct lttng_snapshot_output *output);
+		const struct lttng_snapshot_output *output);
 int cmd_snapshot_record(struct ltt_session *session,
-		struct lttng_snapshot_output *output, int wait);
+		const struct lttng_snapshot_output *output, int wait);
 
 int cmd_set_session_shm_path(struct ltt_session *session,
 		const char *shm_path);
@@ -118,4 +132,17 @@ int cmd_register_trigger(struct command_ctx *cmd_ctx, int sock,
 int cmd_unregister_trigger(struct command_ctx *cmd_ctx, int sock,
 		struct notification_thread_handle *notification_thread_handle);
 
+int cmd_rotate_session(struct ltt_session *session,
+		struct lttng_rotate_session_return *rotate_return,
+		bool quiet_rotation);
+int cmd_rotate_get_info(struct ltt_session *session,
+		struct lttng_rotation_get_info_return *info_return,
+		uint64_t rotate_id);
+int cmd_rotation_set_schedule(struct ltt_session *session,
+		bool activate, enum lttng_rotation_schedule_type schedule_type,
+		uint64_t value,
+		struct notification_thread_handle *notification_thread_handle);
+
+const struct cmd_completion_handler *cmd_pop_completion_handler(void);
+
 #endif /* CMD_H */
