@@ -69,7 +69,7 @@ static void add_unique_ust_event(struct lttng_ht *ht,
  * If event_name is NULL all events of the specified type are disabled.
  */
 int event_kernel_disable_event(struct ltt_kernel_channel *kchan,
-		char *event_name, enum lttng_event_type type)
+		const char *event_name, enum lttng_event_type type)
 {
 	int ret, error = 0, found = 0;
 	struct ltt_kernel_event *kevent;
@@ -119,23 +119,11 @@ int event_kernel_enable_event(struct ltt_kernel_channel *kchan,
 	kevent = trace_kernel_find_event(event->name, kchan,
 			event->type, filter);
 	if (kevent == NULL) {
-		ret = kernel_create_event(event, kchan,
-			filter_expression, filter);
+		ret = kernel_create_event(event, kchan, filter_expression, filter);
 		/* We have passed ownership */
 		filter_expression = NULL;
 		filter = NULL;
-		if (ret < 0) {
-			switch (-ret) {
-			case EEXIST:
-				ret = LTTNG_ERR_KERN_EVENT_EXIST;
-				break;
-			case ENOSYS:
-				ret = LTTNG_ERR_KERN_EVENT_ENOSYS;
-				break;
-			default:
-				ret = LTTNG_ERR_KERN_ENABLE_FAIL;
-				break;
-			}
+		if (ret) {
 			goto end;
 		}
 	} else if (kevent->enabled == 0) {
@@ -184,16 +172,16 @@ int event_ust_enable_tracepoint(struct ltt_ust_session *usess,
 	rcu_read_lock();
 
 	uevent = trace_ust_find_event(uchan->events, event->name, filter,
-			event->loglevel_type, event->loglevel, exclusion);
+			(enum lttng_ust_loglevel_type) event->loglevel_type,
+			event->loglevel, exclusion);
 	if (!uevent) {
-		uevent = trace_ust_create_event(event, filter_expression,
-				filter, exclusion, internal_event);
+		ret = trace_ust_create_event(event, filter_expression,
+				filter, exclusion, internal_event, &uevent);
 		/* We have passed ownership */
 		filter_expression = NULL;
 		filter = NULL;
 		exclusion = NULL;
-		if (uevent == NULL) {
-			ret = LTTNG_ERR_UST_ENABLE_FAIL;
+		if (ret != LTTNG_OK) {
 			goto error;
 		}
 
@@ -203,11 +191,20 @@ int event_ust_enable_tracepoint(struct ltt_ust_session *usess,
 
 	if (uevent->enabled) {
 		/* It's already enabled so everything is OK */
+		assert(!to_create);
 		ret = LTTNG_ERR_UST_EVENT_ENABLED;
 		goto end;
 	}
 
 	uevent->enabled = 1;
+	if (to_create) {
+		/* Add ltt ust event to channel */
+		add_unique_ust_event(uchan->events, uevent);
+	}
+
+	if (!usess->active) {
+		goto end;
+	}
 
 	if (to_create) {
 		/* Create event on all UST registered apps for session */
@@ -227,11 +224,6 @@ int event_ust_enable_tracepoint(struct ltt_ust_session *usess,
 		}
 	}
 
-	if (to_create) {
-		/* Add ltt ust event to channel */
-		add_unique_ust_event(uchan->events, uevent);
-	}
-
 	DBG("Event UST %s %s in channel %s", uevent->attr.name,
 			to_create ? "created" : "enabled", uchan->name);
 
@@ -267,7 +259,7 @@ error:
  * Disable UST tracepoint of a channel from a UST session.
  */
 int event_ust_disable_tracepoint(struct ltt_ust_session *usess,
-		struct ltt_ust_channel *uchan, char *event_name)
+		struct ltt_ust_channel *uchan, const char *event_name)
 {
 	int ret;
 	struct ltt_ust_event *uevent;
@@ -304,17 +296,18 @@ int event_ust_disable_tracepoint(struct ltt_ust_session *usess,
 			/* It's already disabled so everything is OK */
 			goto next;
 		}
+		uevent->enabled = 0;
+		DBG2("Event UST %s disabled in channel %s", uevent->attr.name,
+				uchan->name);
 
+		if (!usess->active) {
+			goto next;
+		}
 		ret = ust_app_disable_event_glb(usess, uchan, uevent);
 		if (ret < 0 && ret != -LTTNG_UST_ERR_EXIST) {
 			ret = LTTNG_ERR_UST_DISABLE_FAIL;
 			goto error;
 		}
-		uevent->enabled = 0;
-
-		DBG2("Event UST %s disabled in channel %s", uevent->attr.name,
-				uchan->name);
-
 next:
 		/* Get next duplicate event by name. */
 		cds_lfht_next_duplicate(ht->ht, trace_ust_ht_match_event_by_name,
@@ -422,6 +415,9 @@ error:
 /*
  * Check if this event's filter requires the activation of application contexts
  * and enable them in the agent.
+ * TODO: bytecode iterator does not support non-legacy application
+ * contexts yet. Not an issue for now, since they are not generated by
+ * the lttng-ctl library.
  */
 static int add_filter_app_ctx(struct lttng_filter_bytecode *bytecode,
 		const char *filter_expression, struct agent *agt)
@@ -515,12 +511,7 @@ int event_agent_enable(struct ltt_ust_session *usess,
 		filter = NULL;
 		filter_expression = NULL;
 		created = 1;
-	}
-
-	/* Already enabled? */
-	if (aevent->enabled) {
-		ret = LTTNG_OK;
-		goto end;
+		assert(!aevent->enabled);
 	}
 
 	if (created && aevent->filter) {
@@ -531,6 +522,12 @@ int event_agent_enable(struct ltt_ust_session *usess,
 		}
 	}
 
+	/* Already enabled? */
+	if (aevent->enabled) {
+		ret = LTTNG_OK;
+		goto end;
+	}
+
 	ret = agent_enable_event(aevent, agt->domain);
 	if (ret != LTTNG_OK) {
 		goto error;
@@ -645,10 +642,12 @@ static int event_agent_disable_one(struct ltt_ust_session *usess,
 	/* If the agent event exists, it must be available on the UST side. */
 	assert(uevent);
 
-	ret = ust_app_disable_event_glb(usess, uchan, uevent);
-	if (ret < 0 && ret != -LTTNG_UST_ERR_EXIST) {
-		ret = LTTNG_ERR_UST_DISABLE_FAIL;
-		goto error;
+	if (usess->active) {
+		ret = ust_app_disable_event_glb(usess, uchan, uevent);
+		if (ret < 0 && ret != -LTTNG_UST_ERR_EXIST) {
+			ret = LTTNG_ERR_UST_DISABLE_FAIL;
+			goto error;
+		}
 	}
 
 	/*
@@ -675,7 +674,7 @@ error:
  * Return LTTNG_OK on success or else a LTTNG_ERR* code.
  */
 int event_agent_disable(struct ltt_ust_session *usess, struct agent *agt,
-		char *event_name)
+		const char *event_name)
 {
 	int ret = LTTNG_OK;
 	struct agent_event *aevent;
