@@ -28,8 +28,11 @@
 
 #include <common/sessiond-comm/sessiond-comm.h>
 #include <common/compat/string.h>
+#include <common/compat/getenv.h>
 #include <common/string-utils/string-utils.h>
+#include <common/utils.h>
 
+#include <lttng/constant.h>
 /* Mi dependancy */
 #include <common/mi-lttng.h>
 
@@ -51,6 +54,7 @@ static int opt_log4j;
 static int opt_python;
 static int opt_enable_all;
 static char *opt_probe;
+static char *opt_userspace_probe;
 static char *opt_function;
 static char *opt_channel_name;
 static char *opt_filter;
@@ -66,6 +70,7 @@ enum {
 	OPT_HELP = 1,
 	OPT_TRACEPOINT,
 	OPT_PROBE,
+	OPT_USERSPACE_PROBE,
 	OPT_FUNCTION,
 	OPT_SYSCALL,
 	OPT_USERSPACE,
@@ -92,6 +97,7 @@ static struct poptOption long_options[] = {
 	{"python",         'p', POPT_ARG_VAL, &opt_python, 1, 0, 0},
 	{"tracepoint",     0,   POPT_ARG_NONE, 0, OPT_TRACEPOINT, 0, 0},
 	{"probe",          0,   POPT_ARG_STRING, &opt_probe, OPT_PROBE, 0, 0},
+	{"userspace-probe",0,   POPT_ARG_STRING, &opt_userspace_probe, OPT_USERSPACE_PROBE, 0, 0},
 	{"function",       0,   POPT_ARG_STRING, &opt_function, OPT_FUNCTION, 0, 0},
 	{"syscall",        0,   POPT_ARG_NONE, 0, OPT_SYSCALL, 0, 0},
 	{"loglevel",       0,     POPT_ARG_STRING, 0, OPT_LOGLEVEL, 0, 0},
@@ -173,6 +179,393 @@ end:
 	return ret;
 }
 
+/*
+ * Walk the directories in the PATH environment variable to find the target
+ * binary passed as parameter.
+ *
+ * On success, the full path of the binary is copied in binary_full_path out
+ * parameter. This buffer is allocated by the caller and must be at least
+ * LTTNG_PATH_MAX bytes long.
+ * On failure, returns -1;
+ */
+static int walk_command_search_path(const char *binary, char *binary_full_path)
+{
+	char *tentative_binary_path = NULL;
+	char *command_search_path = NULL;
+	char *curr_search_dir_end = NULL;
+	char *curr_search_dir = NULL;
+	struct stat stat_output;
+	int ret = 0;
+
+	command_search_path = lttng_secure_getenv("PATH");
+	if (!command_search_path) {
+		ret = -1;
+		goto end;
+	}
+
+	/*
+	 * Duplicate the $PATH string as the char pointer returned by getenv() should
+	 * not be modified.
+	 */
+	command_search_path = strdup(command_search_path);
+	if (!command_search_path) {
+		ret = -1;
+		goto end;
+	}
+
+	/*
+	 * This char array is used to concatenate path to binary to look for
+	 * the binary.
+	 */
+	tentative_binary_path = zmalloc(LTTNG_PATH_MAX * sizeof(char));
+	if (!tentative_binary_path) {
+		ret = -1;
+		goto alloc_error;
+	}
+
+	curr_search_dir = command_search_path;
+	do {
+		/*
+		 * Split on ':'. The return value of this call points to the
+		 * matching character.
+		 */
+		curr_search_dir_end = strchr(curr_search_dir, ':');
+		if (curr_search_dir_end != NULL) {
+			/*
+			 * Add a NULL byte to the end of the first token so it
+			 * can be used as a string.
+			 */
+			curr_search_dir_end[0] = '\0';
+		}
+
+		/* Empty the tentative path */
+		memset(tentative_binary_path, 0, LTTNG_PATH_MAX * sizeof(char));
+
+		/*
+		 * Build the tentative path to the binary using the current
+		 * search directory and the name of the binary.
+		 */
+		ret = snprintf(tentative_binary_path, LTTNG_PATH_MAX, "%s/%s",
+				curr_search_dir, binary);
+		if (ret < 0) {
+			goto free_binary_path;
+		}
+		if (ret < LTTNG_PATH_MAX) {
+			 /*
+			  * Use STAT(2) to see if the file exists.
+			 */
+			ret = stat(tentative_binary_path, &stat_output);
+			if (ret == 0) {
+				/*
+				 * Verify that it is a regular file or a
+				 * symlink and not a special file (e.g.
+				 * device).
+				 */
+				if (S_ISREG(stat_output.st_mode)
+						|| S_ISLNK(stat_output.st_mode)) {
+					/*
+					 * Found a match, set the out parameter
+					 * and return success.
+					 */
+					ret = lttng_strncpy(binary_full_path,
+							tentative_binary_path,
+							LTTNG_PATH_MAX);
+					if (ret == -1) {
+						ERR("Source path does not fit "
+							"in destination buffer.");
+					}
+					goto free_binary_path;
+				}
+			}
+		}
+		/* Go to the next entry in the $PATH variable. */
+		curr_search_dir = curr_search_dir_end + 1;
+	} while (curr_search_dir_end != NULL);
+
+free_binary_path:
+	free(tentative_binary_path);
+alloc_error:
+	free(command_search_path);
+end:
+	return ret;
+}
+
+/*
+ * Check if the symbol field passed by the user is in fact an address or an
+ * offset from a symbol. Those two instrumentation types are not supported yet.
+ * It's expected to be a common mistake because of the existing --probe option
+ * that does support these formats.
+ *
+ * Here are examples of these unsupported formats for the --userspace-probe
+ * option:
+ * elf:/path/to/binary:0x400430
+ * elf:/path/to/binary:4194364
+ * elf:/path/to/binary:my_symbol+0x323
+ * elf:/path/to/binary:my_symbol+43
+ */
+static int warn_userspace_probe_syntax(const char *symbol)
+{
+	int ret;
+
+	/* Check if the symbol field is an hex address. */
+	ret = sscanf(symbol, "0x%*x");
+	if (ret > 0) {
+		/* If there is a match, print a warning and return an error. */
+		ERR("Userspace probe on address not supported yet.");
+		ret = CMD_UNSUPPORTED;
+		goto error;
+	}
+
+	/* Check if the symbol field is an decimal address. */
+	ret = sscanf(symbol, "%*u");
+	if (ret > 0) {
+		/* If there is a match, print a warning and return an error. */
+		ERR("Userspace probe on address not supported yet.");
+		ret = CMD_UNSUPPORTED;
+		goto error;
+	}
+
+	/* Check if the symbol field is symbol+hex_offset. */
+	ret = sscanf(symbol, "%*[^+]+0x%*x");
+	if (ret > 0) {
+		/* If there is a match, print a warning and return an error. */
+		ERR("Userspace probe on symbol+offset not supported yet.");
+		ret = CMD_UNSUPPORTED;
+		goto error;
+	}
+
+	/* Check if the symbol field is symbol+decimal_offset. */
+	ret = sscanf(symbol, "%*[^+]+%*u");
+	if (ret > 0) {
+		/* If there is a match, print a warning and return an error. */
+		ERR("Userspace probe on symbol+offset not supported yet.");
+		ret = CMD_UNSUPPORTED;
+		goto error;
+	}
+
+	ret = 0;
+
+error:
+	return ret;
+}
+
+/*
+ * Parse userspace probe options
+ * Set the userspace probe fields in the lttng_event struct and set the
+ * target_path to the path to the binary.
+ */
+static int parse_userspace_probe_opts(struct lttng_event *ev, char *opt)
+{
+	int ret = CMD_SUCCESS;
+	int num_token;
+	char **tokens;
+	char *target_path = NULL;
+	char *unescaped_target_path = NULL;
+	char *real_target_path = NULL;
+	char *symbol_name = NULL, *probe_name = NULL, *provider_name = NULL;
+	struct lttng_userspace_probe_location *probe_location = NULL;
+	struct lttng_userspace_probe_location_lookup_method *lookup_method =
+			NULL;
+
+	if (opt == NULL) {
+		ret = CMD_ERROR;
+		goto end;
+	}
+
+	switch (ev->type) {
+	case LTTNG_EVENT_USERSPACE_PROBE:
+		break;
+	default:
+		assert(0);
+	}
+
+	/*
+	 * userspace probe fields are separated by ':'.
+	 */
+	tokens = strutils_split(opt, ':', 1);
+	num_token = strutils_array_of_strings_len(tokens);
+
+	/*
+	 * Early sanity check that the number of parameter is between 2 and 4
+	 * inclusively.
+	 * elf:PATH:SYMBOL
+	 * std:PATH:PROVIDER_NAME:PROBE_NAME
+	 * PATH:SYMBOL (same behavior as ELF)
+	 */
+	if (num_token < 2 || num_token > 4) {
+		ret = CMD_ERROR;
+		goto end_string;
+	}
+
+	/*
+	 * Looking up the first parameter will tell the technique to use to
+	 * interpret the userspace probe/function description.
+	 */
+	switch (num_token) {
+	case 2:
+		/* When the probe type is omitted we assume ELF for now. */
+	case 3:
+		if (num_token == 3 && strcmp(tokens[0], "elf") == 0) {
+			target_path = tokens[1];
+			symbol_name = tokens[2];
+		} else if (num_token == 2) {
+			target_path = tokens[0];
+			symbol_name = tokens[1];
+		} else {
+			ret = CMD_ERROR;
+			goto end_string;
+		}
+		lookup_method =
+			lttng_userspace_probe_location_lookup_method_function_elf_create();
+		if (!lookup_method) {
+			WARN("Failed to create ELF lookup method");
+			ret = CMD_ERROR;
+			goto end_string;
+		}
+		break;
+	case 4:
+		if (strcmp(tokens[0], "sdt") == 0) {
+			target_path = tokens[1];
+			provider_name = tokens[2];
+			probe_name = tokens[3];
+		} else {
+			ret = CMD_ERROR;
+			goto end_string;
+		}
+		lookup_method =
+			lttng_userspace_probe_location_lookup_method_tracepoint_sdt_create();
+		if (!lookup_method) {
+			WARN("Failed to create SDT lookup method");
+			ret = CMD_ERROR;
+			goto end_string;
+		}
+		break;
+	default:
+		ret = CMD_ERROR;
+		goto end_string;
+	}
+
+	/* strutils_unescape_string allocates a new char *. */
+	unescaped_target_path = strutils_unescape_string(target_path, 0);
+	if (!unescaped_target_path) {
+		ret = CMD_ERROR;
+		goto end_destroy_lookup_method;
+	}
+
+	/*
+	 * If there is not forward slash in the path. Walk the $PATH else
+	 * expand.
+	 */
+	if (strchr(unescaped_target_path, '/') == NULL) {
+		/* Walk the $PATH variable to find the targeted binary. */
+		real_target_path = zmalloc(LTTNG_PATH_MAX * sizeof(char));
+		if (!real_target_path) {
+			PERROR("Error allocating path buffer");
+			ret = CMD_ERROR;
+			goto end_destroy_lookup_method;
+		}
+		ret = walk_command_search_path(unescaped_target_path, real_target_path);
+		if (ret) {
+			ERR("Binary not found.");
+			ret = CMD_ERROR;
+			goto end_destroy_lookup_method;
+		}
+	} else {
+		/*
+		 * Expand references to `/./` and `/../`. This function does not check
+		 * if the file exists. This call returns an allocated buffer on
+		 * success.
+		 */
+		real_target_path = utils_expand_path_keep_symlink(unescaped_target_path);
+		if (!real_target_path) {
+			ERR("Error expanding the path to binary.");
+			ret = CMD_ERROR;
+			goto end_destroy_lookup_method;
+		}
+
+		/*
+		 * Check if the file exists using access(2), If it does not,
+		 * return an error.
+		 */
+		ret = access(real_target_path, F_OK);
+		if (ret) {
+			ERR("Cannot find binary at path: %s.", real_target_path);
+			ret = CMD_ERROR;
+			goto end_destroy_lookup_method;
+		}
+	}
+
+	switch (lttng_userspace_probe_location_lookup_method_get_type(lookup_method)) {
+	case LTTNG_USERSPACE_PROBE_LOCATION_LOOKUP_METHOD_TYPE_FUNCTION_ELF:
+		/*
+		 * Check for common mistakes in userspace probe description syntax.
+		 */
+		ret = warn_userspace_probe_syntax(symbol_name);
+		if (ret) {
+			goto end_destroy_lookup_method;
+		}
+
+		probe_location = lttng_userspace_probe_location_function_create(
+				real_target_path, symbol_name, lookup_method);
+		if (!probe_location) {
+			WARN("Failed to create function probe location");
+			ret = CMD_ERROR;
+			goto end_destroy_lookup_method;
+		}
+
+		/* Ownership transferred to probe_location. */
+		lookup_method = NULL;
+
+		ret = lttng_event_set_userspace_probe_location(ev, probe_location);
+		if (ret) {
+			WARN("Failed to set probe location on event");
+			ret = CMD_ERROR;
+			goto end_destroy_location;
+		}
+		break;
+	case LTTNG_USERSPACE_PROBE_LOCATION_LOOKUP_METHOD_TYPE_TRACEPOINT_SDT:
+		probe_location = lttng_userspace_probe_location_tracepoint_create(
+				real_target_path, provider_name, probe_name, lookup_method);
+		if (!probe_location) {
+			WARN("Failed to create function probe location");
+			ret = CMD_ERROR;
+			goto end_destroy_lookup_method;
+		}
+
+		/* Ownership transferred to probe_location. */
+		lookup_method = NULL;
+
+		ret = lttng_event_set_userspace_probe_location(ev, probe_location);
+		if (ret) {
+			WARN("Failed to set probe location on event");
+			ret = CMD_ERROR;
+			goto end_destroy_location;
+		}
+		break;
+	default:
+		ret = CMD_ERROR;
+		goto end_destroy_lookup_method;
+	}
+
+	/* Successful parsing, now clean up everything and return. */
+	goto end_string;
+
+end_destroy_location:
+	lttng_userspace_probe_location_destroy(probe_location);
+end_destroy_lookup_method:
+	lttng_userspace_probe_location_lookup_method_destroy(lookup_method);
+end_string:
+	strutils_free_null_terminated_array_of_strings(tokens);
+	/*
+	 * Freeing both char * here makes the error handling simplier. free()
+	 * performs not action if the pointer is NULL.
+	 */
+	free(real_target_path);
+	free(unescaped_target_path);
+end:
+	return ret;
+}
+
 /*
  * Maps LOG4j loglevel from string to value
  */
@@ -412,7 +805,7 @@ char *print_exclusions(char **names)
 {
 	int length = 0;
 	int i;
-	const char *preamble = " excluding ";
+	const char preamble[] = " excluding ";
 	char *ret;
 	int count = names ? strutils_array_of_strings_len(names) : 0;
 
@@ -425,9 +818,8 @@ char *print_exclusions(char **names)
 		length += strlen(names[i]) + 4;
 	}
 
-	/* add length of preamble + one for NUL - one for last (missing) comma */
-	length += strlen(preamble);
-	ret = zmalloc(length + 1);
+	length += sizeof(preamble);
+	ret = zmalloc(length);
 	if (!ret) {
 		return NULL;
 	}
@@ -579,13 +971,18 @@ static int enable_events(char *session_name)
 	int ret = CMD_SUCCESS, command_ret = CMD_SUCCESS;
 	int error_holder = CMD_SUCCESS, warn = 0, error = 0, success = 1;
 	char *event_name, *channel_name = NULL;
-	struct lttng_event ev;
+	struct lttng_event *ev;
 	struct lttng_domain dom;
 	char **exclusion_list = NULL;
 
-	memset(&ev, 0, sizeof(ev));
 	memset(&dom, 0, sizeof(dom));
 
+	ev = lttng_event_create();
+	if (!ev) {
+		ret = CMD_ERROR;
+		goto error;
+	}
+
 	if (opt_kernel) {
 		if (opt_loglevel) {
 			WARN("Kernel loglevels are not supported.");
@@ -635,6 +1032,30 @@ static int enable_events(char *session_name)
 		}
 	}
 
+	/*
+	 * Adding a filter to a probe, function or userspace-probe would be
+	 * denied by the kernel tracer as it's not supported at the moment. We
+	 * do an early check here to warn the user.
+	 */
+	if (opt_filter && opt_kernel) {
+		switch (opt_event_type) {
+		case LTTNG_EVENT_ALL:
+		case LTTNG_EVENT_TRACEPOINT:
+		case LTTNG_EVENT_SYSCALL:
+			break;
+		case LTTNG_EVENT_PROBE:
+		case LTTNG_EVENT_USERSPACE_PROBE:
+		case LTTNG_EVENT_FUNCTION:
+			ERR("Filter expressions are not supported for %s events",
+					get_event_type_str(opt_event_type));
+			ret = CMD_ERROR;
+			goto error;
+		default:
+			ret = CMD_UNDEFINED;
+			goto error;
+		}
+	}
+
 	channel_name = opt_channel_name;
 
 	handle = lttng_create_handle(session_name, &dom);
@@ -656,26 +1077,26 @@ static int enable_events(char *session_name)
 	if (opt_enable_all) {
 		/* Default setup for enable all */
 		if (opt_kernel) {
-			ev.type = opt_event_type;
-			strcpy(ev.name, "*");
+			ev->type = opt_event_type;
+			strcpy(ev->name, "*");
 			/* kernel loglevels not implemented */
-			ev.loglevel_type = LTTNG_EVENT_LOGLEVEL_ALL;
+			ev->loglevel_type = LTTNG_EVENT_LOGLEVEL_ALL;
 		} else {
-			ev.type = LTTNG_EVENT_TRACEPOINT;
-			strcpy(ev.name, "*");
-			ev.loglevel_type = opt_loglevel_type;
+			ev->type = LTTNG_EVENT_TRACEPOINT;
+			strcpy(ev->name, "*");
+			ev->loglevel_type = opt_loglevel_type;
 			if (opt_loglevel) {
 				assert(opt_userspace || opt_jul || opt_log4j || opt_python);
 				if (opt_userspace) {
-					ev.loglevel = loglevel_str_to_value(opt_loglevel);
+					ev->loglevel = loglevel_str_to_value(opt_loglevel);
 				} else if (opt_jul) {
-					ev.loglevel = loglevel_jul_str_to_value(opt_loglevel);
+					ev->loglevel = loglevel_jul_str_to_value(opt_loglevel);
 				} else if (opt_log4j) {
-					ev.loglevel = loglevel_log4j_str_to_value(opt_loglevel);
+					ev->loglevel = loglevel_log4j_str_to_value(opt_loglevel);
 				} else if (opt_python) {
-					ev.loglevel = loglevel_python_str_to_value(opt_loglevel);
+					ev->loglevel = loglevel_python_str_to_value(opt_loglevel);
 				}
-				if (ev.loglevel == -1) {
+				if (ev->loglevel == -1) {
 					ERR("Unknown loglevel %s", opt_loglevel);
 					ret = -LTTNG_ERR_INVALID;
 					goto error;
@@ -683,13 +1104,13 @@ static int enable_events(char *session_name)
 			} else {
 				assert(opt_userspace || opt_jul || opt_log4j || opt_python);
 				if (opt_userspace) {
-					ev.loglevel = -1;
+					ev->loglevel = -1;
 				} else if (opt_jul) {
-					ev.loglevel = LTTNG_LOGLEVEL_JUL_ALL;
+					ev->loglevel = LTTNG_LOGLEVEL_JUL_ALL;
 				} else if (opt_log4j) {
-					ev.loglevel = LTTNG_LOGLEVEL_LOG4J_ALL;
+					ev->loglevel = LTTNG_LOGLEVEL_LOG4J_ALL;
 				} else if (opt_python) {
-					ev.loglevel = LTTNG_LOGLEVEL_PYTHON_DEBUG;
+					ev->loglevel = LTTNG_LOGLEVEL_PYTHON_DEBUG;
 				}
 			}
 		}
@@ -702,13 +1123,13 @@ static int enable_events(char *session_name)
 				goto error;
 			}
 
-			ev.exclusion = 1;
+			ev->exclusion = 1;
 			warn_on_truncated_exclusion_names(exclusion_list,
 				&warn);
 		}
 		if (!opt_filter) {
 			ret = lttng_enable_event_with_exclusions(handle,
-					&ev, channel_name,
+					ev, channel_name,
 					NULL,
 					exclusion_list ? strutils_array_of_strings_len(exclusion_list) : 0,
 					exclusion_list);
@@ -820,7 +1241,7 @@ static int enable_events(char *session_name)
 		}
 
 		if (opt_filter) {
-			command_ret = lttng_enable_event_with_exclusions(handle, &ev, channel_name,
+			command_ret = lttng_enable_event_with_exclusions(handle, ev, channel_name,
 						opt_filter,
 						exclusion_list ? strutils_array_of_strings_len(exclusion_list) : 0,
 						exclusion_list);
@@ -854,7 +1275,7 @@ static int enable_events(char *session_name)
 				}
 				error_holder = command_ret;
 			} else {
-				ev.filter = 1;
+				ev->filter = 1;
 				MSG("Filter '%s' successfully set", opt_filter);
 			}
 		}
@@ -867,16 +1288,16 @@ static int enable_events(char *session_name)
 			 * Note: this is strictly for semantic and printing while in
 			 * machine interface mode.
 			 */
-			strcpy(ev.name, "*");
+			strcpy(ev->name, "*");
 
 			/* If we reach here the events are enabled */
 			if (!error && !warn) {
-				ev.enabled = 1;
+				ev->enabled = 1;
 			} else {
-				ev.enabled = 0;
+				ev->enabled = 0;
 				success = 0;
 			}
-			ret = mi_lttng_event(writer, &ev, 1, handle->domain.type);
+			ret = mi_lttng_event(writer, ev, 1, handle->domain.type);
 			if (ret) {
 				ret = CMD_ERROR;
 				goto error;
@@ -912,9 +1333,9 @@ static int enable_events(char *session_name)
 	event_name = strtok(opt_event_list, ",");
 	while (event_name != NULL) {
 		/* Copy name and type of the event */
-		strncpy(ev.name, event_name, LTTNG_SYMBOL_NAME_LEN);
-		ev.name[LTTNG_SYMBOL_NAME_LEN - 1] = '\0';
-		ev.type = opt_event_type;
+		strncpy(ev->name, event_name, LTTNG_SYMBOL_NAME_LEN);
+		ev->name[LTTNG_SYMBOL_NAME_LEN - 1] = '\0';
+		ev->type = opt_event_type;
 
 		/* Kernel tracer action */
 		if (opt_kernel) {
@@ -925,30 +1346,49 @@ static int enable_events(char *session_name)
 			switch (opt_event_type) {
 			case LTTNG_EVENT_ALL:	/* Enable tracepoints and syscalls */
 				/* If event name differs from *, select tracepoint. */
-				if (strcmp(ev.name, "*")) {
-					ev.type = LTTNG_EVENT_TRACEPOINT;
+				if (strcmp(ev->name, "*")) {
+					ev->type = LTTNG_EVENT_TRACEPOINT;
 				}
 				break;
 			case LTTNG_EVENT_TRACEPOINT:
 				break;
 			case LTTNG_EVENT_PROBE:
-				ret = parse_probe_opts(&ev, opt_probe);
+				ret = parse_probe_opts(ev, opt_probe);
 				if (ret) {
 					ERR("Unable to parse probe options");
-					ret = 0;
+					ret = CMD_ERROR;
+					goto error;
+				}
+				break;
+			case LTTNG_EVENT_USERSPACE_PROBE:
+				ret = parse_userspace_probe_opts(ev, opt_userspace_probe);
+				if (ret) {
+					switch (ret) {
+					case CMD_UNSUPPORTED:
+						/*
+						 * Error message describing
+						 * what is not supported was
+						 * printed in the function.
+						 */
+						break;
+					case CMD_ERROR:
+					default:
+						ERR("Unable to parse userspace probe options");
+						break;
+					}
 					goto error;
 				}
 				break;
 			case LTTNG_EVENT_FUNCTION:
-				ret = parse_probe_opts(&ev, opt_function);
+				ret = parse_probe_opts(ev, opt_function);
 				if (ret) {
 					ERR("Unable to parse function probe options");
-					ret = 0;
+					ret = CMD_ERROR;
 					goto error;
 				}
 				break;
 			case LTTNG_EVENT_SYSCALL:
-				ev.type = LTTNG_EVENT_SYSCALL;
+				ev->type = LTTNG_EVENT_SYSCALL;
 				break;
 			default:
 				ret = CMD_UNDEFINED;
@@ -956,7 +1396,7 @@ static int enable_events(char *session_name)
 			}
 
 			/* kernel loglevels not implemented */
-			ev.loglevel_type = LTTNG_EVENT_LOGLEVEL_ALL;
+			ev->loglevel_type = LTTNG_EVENT_LOGLEVEL_ALL;
 		} else if (opt_userspace) {		/* User-space tracer action */
 			DBG("Enabling UST event %s for channel %s, loglevel %s", event_name,
 					print_channel_name(channel_name), opt_loglevel ? : "<all>");
@@ -966,13 +1406,14 @@ static int enable_events(char *session_name)
 				/* Fall-through */
 			case LTTNG_EVENT_TRACEPOINT:
 				/* Copy name and type of the event */
-				ev.type = LTTNG_EVENT_TRACEPOINT;
-				strncpy(ev.name, event_name, LTTNG_SYMBOL_NAME_LEN);
-				ev.name[LTTNG_SYMBOL_NAME_LEN - 1] = '\0';
+				ev->type = LTTNG_EVENT_TRACEPOINT;
+				strncpy(ev->name, event_name, LTTNG_SYMBOL_NAME_LEN);
+				ev->name[LTTNG_SYMBOL_NAME_LEN - 1] = '\0';
 				break;
 			case LTTNG_EVENT_PROBE:
 			case LTTNG_EVENT_FUNCTION:
 			case LTTNG_EVENT_SYSCALL:
+			case LTTNG_EVENT_USERSPACE_PROBE:
 			default:
 				ERR("Event type not available for user-space tracing");
 				ret = CMD_UNSUPPORTED;
@@ -980,7 +1421,7 @@ static int enable_events(char *session_name)
 			}
 
 			if (opt_exclude) {
-				ev.exclusion = 1;
+				ev->exclusion = 1;
 				if (opt_event_type != LTTNG_EVENT_ALL && opt_event_type != LTTNG_EVENT_TRACEPOINT) {
 					ERR("Exclusion option can only be used with tracepoint events");
 					ret = CMD_ERROR;
@@ -1002,16 +1443,16 @@ static int enable_events(char *session_name)
 					exclusion_list, &warn);
 			}
 
-			ev.loglevel_type = opt_loglevel_type;
+			ev->loglevel_type = opt_loglevel_type;
 			if (opt_loglevel) {
-				ev.loglevel = loglevel_str_to_value(opt_loglevel);
-				if (ev.loglevel == -1) {
+				ev->loglevel = loglevel_str_to_value(opt_loglevel);
+				if (ev->loglevel == -1) {
 					ERR("Unknown loglevel %s", opt_loglevel);
 					ret = -LTTNG_ERR_INVALID;
 					goto error;
 				}
 			} else {
-				ev.loglevel = -1;
+				ev->loglevel = -1;
 			}
 		} else if (opt_jul || opt_log4j || opt_python) {
 			if (opt_event_type != LTTNG_EVENT_ALL &&
@@ -1021,32 +1462,32 @@ static int enable_events(char *session_name)
 				goto error;
 			}
 
-			ev.loglevel_type = opt_loglevel_type;
+			ev->loglevel_type = opt_loglevel_type;
 			if (opt_loglevel) {
 				if (opt_jul) {
-					ev.loglevel = loglevel_jul_str_to_value(opt_loglevel);
+					ev->loglevel = loglevel_jul_str_to_value(opt_loglevel);
 				} else if (opt_log4j) {
-					ev.loglevel = loglevel_log4j_str_to_value(opt_loglevel);
+					ev->loglevel = loglevel_log4j_str_to_value(opt_loglevel);
 				} else if (opt_python) {
-					ev.loglevel = loglevel_python_str_to_value(opt_loglevel);
+					ev->loglevel = loglevel_python_str_to_value(opt_loglevel);
 				}
-				if (ev.loglevel == -1) {
+				if (ev->loglevel == -1) {
 					ERR("Unknown loglevel %s", opt_loglevel);
 					ret = -LTTNG_ERR_INVALID;
 					goto error;
 				}
 			} else {
 				if (opt_jul) {
-					ev.loglevel = LTTNG_LOGLEVEL_JUL_ALL;
+					ev->loglevel = LTTNG_LOGLEVEL_JUL_ALL;
 				} else if (opt_log4j) {
-					ev.loglevel = LTTNG_LOGLEVEL_LOG4J_ALL;
+					ev->loglevel = LTTNG_LOGLEVEL_LOG4J_ALL;
 				} else if (opt_python) {
-					ev.loglevel = LTTNG_LOGLEVEL_PYTHON_DEBUG;
+					ev->loglevel = LTTNG_LOGLEVEL_PYTHON_DEBUG;
 				}
 			}
-			ev.type = LTTNG_EVENT_TRACEPOINT;
-			strncpy(ev.name, event_name, LTTNG_SYMBOL_NAME_LEN);
-			ev.name[LTTNG_SYMBOL_NAME_LEN - 1] = '\0';
+			ev->type = LTTNG_EVENT_TRACEPOINT;
+			strncpy(ev->name, event_name, LTTNG_SYMBOL_NAME_LEN);
+			ev->name[LTTNG_SYMBOL_NAME_LEN - 1] = '\0';
 		} else {
 			assert(0);
 		}
@@ -1055,7 +1496,7 @@ static int enable_events(char *session_name)
 			char *exclusion_string;
 
 			command_ret = lttng_enable_event_with_exclusions(handle,
-					&ev, channel_name,
+					ev, channel_name,
 					NULL,
 					exclusion_list ? strutils_array_of_strings_len(exclusion_list) : 0,
 					exclusion_list);
@@ -1086,6 +1527,12 @@ static int enable_events(char *session_name)
 					error = 1;
 					break;
 				}
+				case LTTNG_ERR_SDT_PROBE_SEMAPHORE:
+					ERR("SDT probes %s guarded by semaphores are not supported (channel %s, session %s)",
+							event_name, print_channel_name(channel_name),
+							session_name);
+					error = 1;
+					break;
 				default:
 					ERR("Event %s%s: %s (channel %s, session %s)", event_name,
 							exclusion_string,
@@ -1131,9 +1578,9 @@ static int enable_events(char *session_name)
 			char *exclusion_string;
 
 			/* Filter present */
-			ev.filter = 1;
+			ev->filter = 1;
 
-			command_ret = lttng_enable_event_with_exclusions(handle, &ev, channel_name,
+			command_ret = lttng_enable_event_with_exclusions(handle, ev, channel_name,
 					opt_filter,
 					exclusion_list ? strutils_array_of_strings_len(exclusion_list) : 0,
 					exclusion_list);
@@ -1156,7 +1603,7 @@ static int enable_events(char *session_name)
 				case LTTNG_ERR_TRACE_ALREADY_STARTED:
 				{
 					const char *msg = "The command tried to enable an event in a new domain for a session that has already been started once.";
-					ERR("Event %s%s: %s (channel %s, session %s, filter \'%s\')", ev.name,
+					ERR("Event %s%s: %s (channel %s, session %s, filter \'%s\')", ev->name,
 							exclusion_string,
 							msg,
 							print_channel_name(channel_name),
@@ -1165,7 +1612,7 @@ static int enable_events(char *session_name)
 					break;
 				}
 				default:
-					ERR("Event %s%s: %s (channel %s, session %s, filter \'%s\')", ev.name,
+					ERR("Event %s%s: %s (channel %s, session %s, filter \'%s\')", ev->name,
 							exclusion_string,
 							lttng_strerror(command_ret),
 							command_ret == -LTTNG_ERR_NEED_CHANNEL_NAME
@@ -1188,12 +1635,12 @@ static int enable_events(char *session_name)
 		if (lttng_opt_mi) {
 			if (command_ret) {
 				success = 0;
-				ev.enabled = 0;
+				ev->enabled = 0;
 			} else {
-				ev.enabled = 1;
+				ev->enabled = 1;
 			}
 
-			ret = mi_lttng_event(writer, &ev, 1, handle->domain.type);
+			ret = mi_lttng_event(writer, ev, 1, handle->domain.type);
 			if (ret) {
 				ret = CMD_ERROR;
 				goto error;
@@ -1253,6 +1700,7 @@ error:
 	 */
 	ret = error_holder ? error_holder : ret;
 
+	lttng_event_destroy(ev);
 	return ret;
 }
 
@@ -1284,6 +1732,9 @@ int cmd_enable_events(int argc, const char **argv)
 		case OPT_PROBE:
 			opt_event_type = LTTNG_EVENT_PROBE;
 			break;
+		case OPT_USERSPACE_PROBE:
+			opt_event_type = LTTNG_EVENT_USERSPACE_PROBE;
+			break;
 		case OPT_FUNCTION:
 			opt_event_type = LTTNG_EVENT_FUNCTION;
 			break;
