@@ -134,6 +134,7 @@ static void test_create_ust_event(void)
 {
 	struct ltt_ust_event *event;
 	struct lttng_event ev;
+	enum lttng_error_code ret;
 
 	memset(&ev, 0, sizeof(ev));
 	ok(lttng_strncpy(ev.name, get_random_string(),
@@ -142,9 +143,9 @@ static void test_create_ust_event(void)
 	ev.type = LTTNG_EVENT_TRACEPOINT;
 	ev.loglevel_type = LTTNG_EVENT_LOGLEVEL_ALL;
 
-	event = trace_ust_create_event(&ev, NULL, NULL, NULL, false);
+	ret = trace_ust_create_event(&ev, NULL, NULL, NULL, false, &event);
 
-	ok(event != NULL, "Create UST event");
+	ok(ret == LTTNG_OK, "Create UST event");
 
 	if (!event) {
 		skip(1, "UST event is null");
@@ -162,11 +163,13 @@ static void test_create_ust_event(void)
 
 static void test_create_ust_event_exclusion(void)
 {
+	enum lttng_error_code ret;
 	struct ltt_ust_event *event;
 	struct lttng_event ev;
 	char *name;
 	char *random_name;
-	struct lttng_event_exclusion *exclusion;
+	struct lttng_event_exclusion *exclusion = NULL;
+	struct lttng_event_exclusion *exclusion_copy = NULL;
 	const int exclusion_count = 2;
 
 	memset(&ev, 0, sizeof(ev));
@@ -196,10 +199,10 @@ static void test_create_ust_event_exclusion(void)
 	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 1), random_name,
 		LTTNG_SYMBOL_NAME_LEN);
 
-	event = trace_ust_create_event(&ev, NULL, NULL, exclusion, false);
+	ret = trace_ust_create_event(&ev, NULL, NULL, exclusion, false, &event);
 	exclusion = NULL;
 
-	ok(!event, "Create UST event with identical exclusion names fails");
+	ok(ret != LTTNG_OK, "Create UST event with identical exclusion names fails");
 
 	exclusion = zmalloc(sizeof(*exclusion) +
 		LTTNG_SYMBOL_NAME_LEN * exclusion_count);
@@ -209,14 +212,34 @@ static void test_create_ust_event_exclusion(void)
 		goto end;
 	}
 
+	exclusion_copy = zmalloc(sizeof(*exclusion) +
+		LTTNG_SYMBOL_NAME_LEN * exclusion_count);
+	if (!exclusion_copy) {
+		skip(2, "zmalloc failed");
+		goto end;
+	}
+
+	/*
+	 * We are giving ownership of the exclusion struct to the
+	 * trace_ust_create_event() function. Make a copy of the exclusion struct
+	 * so we can compare it later.
+	 */
+
 	exclusion->count = exclusion_count;
 	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 0),
 		get_random_string(), LTTNG_SYMBOL_NAME_LEN);
 	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 1),
 		get_random_string(), LTTNG_SYMBOL_NAME_LEN);
 
-	event = trace_ust_create_event(&ev, NULL, NULL, exclusion, false);
-	ok(event != NULL, "Create UST event with different exclusion names");
+	exclusion_copy->count = exclusion_count;
+	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion_copy, 0),
+		LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 0), LTTNG_SYMBOL_NAME_LEN);
+	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion_copy, 1),
+		LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 1), LTTNG_SYMBOL_NAME_LEN);
+
+	ret = trace_ust_create_event(&ev, NULL, NULL, exclusion, false, &event);
+	exclusion = NULL;
+	ok(ret == LTTNG_OK, "Create UST event with different exclusion names");
 
 	if (!event) {
 		skip(1, "UST event with exclusion is null");
@@ -224,17 +247,19 @@ static void test_create_ust_event_exclusion(void)
 	}
 
 	ok(event->enabled == 0 &&
-	   event->attr.instrumentation == LTTNG_UST_TRACEPOINT &&
-	   strcmp(event->attr.name, ev.name) == 0 &&
-	   event->exclusion != NULL &&
-	   event->exclusion->count == exclusion_count &&
-	   !memcmp(event->exclusion->names, exclusion->names,
-	   	LTTNG_SYMBOL_NAME_LEN * exclusion_count) &&
-	   event->attr.name[LTTNG_UST_SYM_NAME_LEN - 1] == '\0',
-	   "Validate UST event and exclusion");
+		event->attr.instrumentation == LTTNG_UST_TRACEPOINT &&
+		strcmp(event->attr.name, ev.name) == 0 &&
+		event->exclusion != NULL &&
+		event->exclusion->count == exclusion_count &&
+		!memcmp(event->exclusion->names, exclusion_copy->names,
+			LTTNG_SYMBOL_NAME_LEN * exclusion_count) &&
+		event->attr.name[LTTNG_UST_SYM_NAME_LEN - 1] == '\0',
+		"Validate UST event and exclusion");
 
 	trace_ust_destroy_event(event);
 end:
+	free(exclusion);
+	free(exclusion_copy);
 	return;
 }
 
